from acex.configuration.components.base_component import ConfigComponent
from acex.configuration.components.interfaces import Interface
from acex_devkit.models.composed_configuration import (
    aaaTacacsAttributes,
    aaaRadiusAttributes,
    aaaServerGroupAttributes,
    aaaAuthenticationMethods as aaaAuthenticationMethodsAttributes,
    aaaAuthorizationMethods as aaaAuthorizationMethodsAttributes,
    aaaAccountingMethods as aaaAccountingMethodsAttributes,
    ReferenceTo
)

class aaaTacacs(ConfigComponent):
    type = "aaaTacacs"
    model_cls = aaaTacacsAttributes

    #def pre_init(self):
    #    # Resolve source_interface
    #    if "source_interface" in self.kwargs:
    #        si = self.kwargs.pop("source_interface")
    #        if isinstance(si, type(None)):
    #            pass
    #        elif isinstance(si, str):
    #            ref = ReferenceTo(pointer=f"interfaces.{si}")
    #            self.kwargs["source_interface"] = ref
#
    #        elif isinstance(si, Interface):
    #            ref = ReferenceTo(pointer=f"interfaces.{si.name}")
    #            self.kwargs["source_interface"] = ref

class aaaRadius(ConfigComponent):
    type = "aaaRadius"
    model_cls = aaaRadiusAttributes

    #def pre_init(self):
    #    # Resolve source_interface
    #    if "source_interface" in self.kwargs:
    #        si = self.kwargs.pop("source_interface")
    #        if isinstance(si, type(None)):
    #            pass
    #        elif isinstance(si, str):
    #            ref = ReferenceTo(pointer=f"interfaces.{si}")
    #            self.kwargs["source_interface"] = ref
#
    #        elif isinstance(si, Interface):
    #            ref = ReferenceTo(pointer=f"interfaces.{si.name}")
    #            self.kwargs["source_interface"] = ref

class aaaServerGroup(ConfigComponent):
    type = "aaaServerGroup"
    model_cls = aaaServerGroupAttributes

    def pre_init(self):
        # Resolve tacacs
        if "tacacs" in self.kwargs:
            tacacs = self.kwargs.pop("tacacs")
            if isinstance(tacacs, type(None)):
                pass

            elif isinstance(tacacs, aaaTacacs):
                ref = ReferenceTo(pointer=f"system.aaa.tacacs.{tacacs.name}")
                self.kwargs["tacacs"] = ref
                
        if "radius" in self.kwargs:
            radius = self.kwargs.pop("radius")
            if isinstance(radius, type(None)):
                pass

            elif isinstance(radius, aaaRadius):
                ref = ReferenceTo(pointer=f"system.aaa.radius.{radius.name}")
                self.kwargs["radius"] = ref

class aaaAuthenticationMethods(ConfigComponent):
    type = "aaaAuthenticationMethods"
    model_cls = aaaAuthenticationMethodsAttributes

class aaaAuthorizationMethods(ConfigComponent):
    type = "aaaAuthorizationMethods"
    model_cls = aaaAuthorizationMethodsAttributes

class aaaAccountingMethods(ConfigComponent):
    type = "aaaAccountingMethods"
    model_cls = aaaAccountingMethodsAttributes



from .adapter_base import AdapterBase
from acex.models import Asset, AssetResponse


class AssetAdapter(AdapterBase):

    def create(self, asset: Asset): 
        if hasattr(self.plugin, "create"):
            return getattr(self.plugin, "create")(asset)

    def get(self, id: str) -> AssetResponse: 
        if hasattr(self.plugin, "get"):
            return getattr(self.plugin, "get")(id)

    def query(self) -> list[AssetResponse]: 
        if hasattr(self.plugin, "query"):
            return getattr(self.plugin, "query")()

    def update(self, id: str, asset: Asset): 
        if hasattr(self.plugin, "update"):
            return getattr(self.plugin, "update")(id, asset)

    def delete(self, id: str): 
        if hasattr(self.plugin, "delete"):
            return getattr(self.plugin, "delete")(id)





from acex.models.asset import AssetClusterCreate, AssetCluster, AssetClusterLink
from fastapi import HTTPException

class AssetClusterManager:

    def __init__(self, db_manager):
        self.db = db_manager

    def create_cluster(self, payload: AssetClusterCreate) -> AssetCluster:
        session = next(self.db.get_session())
        try:
            cluster = AssetCluster.model_validate(payload)
            session.add(cluster)
            session.commit()
            session.refresh(cluster)
            return cluster
        finally:
            session.close()

    def list_clusters(self) -> list[AssetCluster]:
        session = next(self.db.get_session())
        try:
            return session.query(AssetCluster).all()
        finally:
            session.close()

    def get_cluster(self, id: int) -> dict:
        session = next(self.db.get_session())
        try:
            cluster = session.get(AssetCluster, id)
            if not cluster:
                raise HTTPException(status_code=404, detail="AssetCluster not found")

            # Hämta alla AssetClusterLink för detta kluster
            links = session.query(AssetClusterLink).filter(AssetClusterLink.cluster_id == id).all()
            # Skapa en mapping asset_id -> order
            order_map = {link.asset_id: link.order for link in links}

            # Skapa och sortera asset-listan med cluster_index
            assets = [
                {
                    "id": asset.id,
                    "vendor": asset.vendor,
                    "serial_number": asset.serial_number,
                    "os": asset.os,
                    "os_version": asset.os_version,
                    "hardware_model": asset.hardware_model,
                    "ned_id": asset.ned_id,
                    "cluster_index": order_map.get(asset.id)
                }
                for asset in cluster.assets
            ]
            assets.sort(key=lambda a: (a["cluster_index"] if a["cluster_index"] is not None else 0))

            return {
                "id": cluster.id,
                "name": cluster.name,
                "assets": assets
            }
        finally:
            session.close()

    def update_cluster_assets(self, id: int, asset_ids: list[int]) -> AssetCluster:
        session = next(self.db.get_session())
        try:
            cluster = session.get(AssetCluster, id)
            if not cluster:
                raise HTTPException(status_code=404, detail="AssetCluster not found")

            # Ta bort gamla länkar
            session.exec(
                AssetClusterLink.__table__.delete().where(AssetClusterLink.cluster_id == id)
            )

            # Lägg till nya länkar
            for order, asset_id in enumerate(asset_ids):
                link = AssetClusterLink(asset_id=asset_id, cluster_id=id, order=order)
                session.add(link)

            session.commit()
            session.refresh(cluster)
            return cluster
        finally:
            session.close()

    def delete_cluster(self, id: int) -> None:
        session = next(self.db.get_session())
        try:
            cluster = session.get(AssetCluster, id)
            if not cluster:
                raise HTTPException(status_code=404, detail="AssetCluster not found")
            session.delete(cluster)
            session.commit()
        finally:
            session.close()